#!perl
package File::Replace::Fancy;
use warnings;
use strict;
use File::Replace::SingleHandle ();
use File::Replace::DualHandle ();

=head1 Name

File::Replace::Fancy - Perl extension that extends the File::Replace module

=head1 Description

**This is a stub module.** You don't need to C<use> it.
Please see L<File::Replace> for the features provided by this distribution.

=head1 Author, Copyright, and License

Copyright (c) 2017-2023 Hauke Daempfling (haukex@zero-g.net)
at the Leibniz Institute of Freshwater Ecology and Inland Fisheries (IGB),
Berlin, Germany, L<http://www.igb-berlin.de/>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see L<http://www.gnu.org/licenses/>.

=cut

1;
